#!/usr/bin/perl

use POSIX qw(strftime);

require "global.conf"
    or &fatalerror("Cannot find global configuration file - global.conf");

$scriptlog="backup2email.log";	# Where to write script logs.

&line2log("-"x30);		# Separate records in script log file.

# If no arguments given we simply should die...
if (!@ARGV) {
    $luckyjobs=&findvalidjobs();
    &fatalerror("No job(s) specified. Possible job values: $luckyjobs");
}

# We can get more then one backup job from the command line, so better do them
# all together.
foreach $backupjob (@ARGV) {
    if (-e "$backupjob.conf") {
	require "$backupjob.conf";	# Job specific configuration file.
    } 

    $tempfile="$archivedir/$backupjob/lastsent.txt";

    &line2log("START: Job # $backupjob");
    $backuplogfile=&findbackuplog();
    &checkifnew();
    $archivejobdir="$archivedir/$backupjob";
    &createdir($archivedir,$archivejobdir);
    ($admreport,$mngreport)=&archivelogfile();
    # If admin's/amdinassistant's email is specified, then send full report
    if ($admin || $admassistent) {
	&sendreport("adm");		
    }
    # If manager's/managerassistant's email is specifed, then send short report
    if ($manager || $mngassistent) {
	&sendreport("mng");
    }
    &line2log("END: Job # $backupjob");
}
exit(0);			# Normal script termination
###############################################################################
sub findvalidjobs {
    # This script tryies to find valid jobs in the NovaNet backup directory.
    @alljobs=`ls -1 $backupdir`;
    foreach $job (@alljobs) {
	chomp($job);
	if ($job=~s/nn(([a-f]|[0-9]){6})\.([a-e]|[0-9]){1}([a-f]|[0-9]){2}/$1/) {
	    $jobs{$job}++;
	}
    }
    foreach $key (sort keys %jobs) { $validjobs.=$key.";"; &line2log("DEBUG: $key"); }
    return $validjobs;
}
sub findbackuplog {
    # This sub is intended to find the last log file in the NovaNet database
    # directory.  
    @backuplogs=`ls -1 $backupdir | grep $backupjob`;
    foreach $backuplog (@backuplogs) {
	chomp($backuplog);
	if ($backuplog=~/nn$backupjob\.([a-e]|[0-9]){1}([a-f]|[0-9]){2}/) {
	    $lastbackuplog=$backuplog;
	}
    }
    $lastbackuplog="$backupdir/$lastbackuplog";
    &line2log("Last log filename is $lastbackuplog");
    return $lastbackuplog;
}

sub checkifnew {
    # This sub is supposed to check if the received log file had already been
    # sent to the admins and/or managers.  If it was, then script terminates
    # normally.
    if (-e $tempfile) {
	open (TEMP, $tempfile)
	    or &fatalerror("Could not read from $tempfile ($!)");
	chomp($lastsent=<TEMP>);	# Read filename of last log file sent
	close (TEMP);
	if ($lastsent eq $backuplogfile) {
	    &line2log("$backuplogfile has already been sent");
	    &line2log("END: Job # $backupjob");
	    exit(0);
	}
    }
}

sub archivelogfile {
    # This sub archives the NovaNet log file in our archieves/JOBNUMBER 
    # directory, as well generates the shortened report file for the manager
    # (basically all lines containing word "Job").
    $currenttime=strftime "%Y-%m-%d-%H%M%S", localtime(time);
    $fulllog="$archivejobdir/$currenttime-full.log";
    $shortlog="$archivejobdir/$currenttime-short.log";
    open(IN, "$backuplogfile") 
	or &fatalerror("Could not read from $backuplogfile ($!)");
    open (OUTLONG, ">$fulllog")
	or &fatalerror("Could not write to $fulllog ($!)");
    open (OUTSHORT, ">$shortlog")
	or &fatalerror("Could not write to $shortlog ($!)");
    while ($linein=<IN>) {
	print OUTLONG $linein;
	if ($linein=~/Job/) {
	    print OUTSHORT $linein;
	}
    }
    close (OUTSHORT);
    &line2log("Generated manager report file $shortlog");
    close (OUTLONG);
    &line2log("Generated admin report file $fulllog");
    close(IN);
    open (TEMP, ">$tempfile")
	or &fatalerror("Could not write to $tempfile ($!)");
    print TEMP "$backuplogfile\n";
    close(TEMP);
    return $fulllog,$shortlog;
}

sub sendreport {
    # This sub sends the report file to the appropriate addresses with the
    # appropriate subject.
    $reporttype=shift;
    if ($reporttype eq "adm") {
	&line2log("Sending admin's report");
        $admsubject=&processmacro($admsubject);
	$sendmail="$mailcommand '$admin' -s '$admsubject' -c '$admassistent' <  $admreport";
    } elsif ($reporttype eq "mng") {
	&line2log("Sending manager's report");
        $mngsubject=&processmacro($mngsubject);
	$sendmail="$mailcommand '$manager' -s '$mngsubject' -c '$mngassistent' < $mngreport";
    } else {
	&fatalerror("Report type is not specified [adm/mng]");
    }
    system ($sendmail);
}

sub processmacro {
    # This sub substitutes the appropriate values for the macros in the string.
    $textline=shift;
    $subjdate=strftime "%Y-%m-%d", localtime(time);	# Get current date.
    $subjtime=strftime "%H:%M:%S", localtime(time);	# Get current time.
    $textline=~s/\<job\>/$backupjob/g;			# Process <job> macro.
    $textline=~s/\<date\>/$subjdate/g;			# Process <date> macro.
    $textline=~s/\<time\>/$subjtime/g;			# Process <time> macro.
    return $textline;
}

sub createdir {
    # This sub creates directory(s) specified in the arguments, if they do
    # not exist.  Default permissions: rwxrwx---.
    while ($dirname=shift) {
	if (!(-e $dirname)) { 
	    mkdir ($dirname,0770) 
		or &fatalerror("Couldn't create directory $dirname($!)"); 
	    &line2log("Created directory $dirname");
	}
    }
}

sub line2log {
    # This sub appends recieved string to the log file of the script itself,
    # and adds current time aswell.
    $message=shift;
    $msgtime=strftime "%H:%M:%S", localtime(time);
    open (LOG, ">>$scriptlog");
	print LOG "$msgtime>  $message\n";
    close (LOG);
}

sub fatalerror {
    # This sub is used when any fatal error happens.  Logs the error, prints it
    # on the screen and exits with abnormal exit code.
    $errortext=shift;
    &line2log("FATAL: $errortext");
    &line2log("CRASHED");
    print "\n$errortext\n\n";
    exit (1);
}
